from pyasic.miners.makes import AuradineMake


class AuradineAT2860(AuradineMake):
    raw_model = "AT2860"
    expected_fans = 4


class AuradineAT2880(AuradineMake):
    raw_model = "AT2880"
    expected_fans = 4
