# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------

from pyasic.miners.makes import WhatsMinerMake


class M31SPlusV10(WhatsMinerMake):
    raw_model = "M31S+ V10"
    expected_chips = 105
    expected_fans = 2


class M31SPlusV20(WhatsMinerMake):
    raw_model = "M31S+ V20"
    expected_chips = 111
    expected_fans = 2


class M31SPlusV30(WhatsMinerMake):
    raw_model = "M31S+ V30"
    expected_chips = 117
    expected_fans = 2


class M31SPlusV40(WhatsMinerMake):
    raw_model = "M31S+ V40"
    expected_chips = 123
    expected_fans = 2


class M31SPlusV50(WhatsMinerMake):
    raw_model = "M31S+ V50"
    expected_chips = 148
    expected_fans = 2


class M31SPlusV60(WhatsMinerMake):
    raw_model = "M31S+ V60"
    expected_chips = 156
    expected_fans = 2


class M31SPlusV80(WhatsMinerMake):
    raw_model = "M31S+ V80"
    expected_chips = 129
    expected_fans = 2


class M31SPlusV90(WhatsMinerMake):
    raw_model = "M31S+ V90"
    expected_chips = 117
    expected_fans = 2


class M31SPlusV100(WhatsMinerMake):
    raw_model = "M31S+ V100"
    expected_chips = 111
    expected_fans = 2


class M31SPlusVE10(WhatsMinerMake):
    raw_model = "M31S+ VE10"
    expected_chips = 82
    expected_fans = 2


class M31SPlusVE20(WhatsMinerMake):
    raw_model = "M31S+ VE20"
    expected_chips = 78
    expected_fans = 2


class M31SPlusVE30(WhatsMinerMake):
    raw_model = "M31S+ VE30"
    expected_chips = 105
    expected_fans = 2


class M31SPlusVE40(WhatsMinerMake):
    raw_model = "M31S+ VE40"
    expected_chips = 111
    expected_fans = 2


class M31SPlusVE50(WhatsMinerMake):
    raw_model = "M31S+ VE50"
    expected_chips = 117
    expected_fans = 2


class M31SPlusVE60(WhatsMinerMake):
    raw_model = "M31S+ VE60"
    expected_fans = 2


class M31SPlusVE80(WhatsMinerMake):
    raw_model = "M31S+ VE80"
    expected_fans = 2


class M31SPlusVF20(WhatsMinerMake):
    raw_model = "M31S+ VF20"
    expected_chips = 66
    expected_fans = 2


class M31SPlusVF30(WhatsMinerMake):
    raw_model = "M31S+ VF30"
    expected_fans = 2


class M31SPlusVG20(WhatsMinerMake):
    raw_model = "M31S+ VG20"
    expected_chips = 66
    expected_fans = 2


class M31SPlusVG30(WhatsMinerMake):
    raw_model = "M31S+ VG30"
    expected_chips = 70
    expected_fans = 2
