# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------

from pyasic.miners.makes import WhatsMinerMake


class M30SV10(WhatsMinerMake):
    raw_model = "M30S V10"
    expected_chips = 148
    expected_fans = 2


class M30SV20(WhatsMinerMake):
    raw_model = "M30S V20"
    expected_chips = 156
    expected_fans = 2


class M30SV30(WhatsMinerMake):
    raw_model = "M30S V30"
    expected_chips = 164
    expected_fans = 2


class M30SV40(WhatsMinerMake):
    raw_model = "M30S V40"
    expected_chips = 172
    expected_fans = 2


class M30SV50(WhatsMinerMake):
    raw_model = "M30S V50"
    expected_chips = 156
    expected_fans = 2


class M30SV60(WhatsMinerMake):
    raw_model = "M30S V60"
    expected_chips = 164
    expected_fans = 2


class M30SV70(WhatsMinerMake):
    raw_model = "M30S V70"
    expected_fans = 2


class M30SV80(WhatsMinerMake):
    raw_model = "M30S V80"
    expected_chips = 129
    expected_fans = 2


class M30SVE10(WhatsMinerMake):
    raw_model = "M30S VE10"
    expected_chips = 105
    expected_fans = 2


class M30SVE20(WhatsMinerMake):
    raw_model = "M30S VE20"
    expected_chips = 111
    expected_fans = 2


class M30SVE30(WhatsMinerMake):
    raw_model = "M30S VE30"
    expected_chips = 117
    expected_fans = 2


class M30SVE40(WhatsMinerMake):
    raw_model = "M30S VE40"
    expected_chips = 123
    expected_fans = 2


class M30SVE50(WhatsMinerMake):
    raw_model = "M30S VE50"
    expected_chips = 129
    expected_fans = 2


class M30SVE60(WhatsMinerMake):
    raw_model = "M30S VE60"
    expected_fans = 2


class M30SVE70(WhatsMinerMake):
    raw_model = "M30S VE70"
    expected_fans = 2


class M30SVF10(WhatsMinerMake):
    raw_model = "M30S VF10"
    expected_chips = 70
    expected_fans = 2


class M30SVF20(WhatsMinerMake):
    raw_model = "M30S VF20"
    expected_chips = 74
    expected_fans = 2


class M30SVF30(WhatsMinerMake):
    raw_model = "M30S VF30"
    expected_chips = 78
    expected_fans = 2


class M30SVG10(WhatsMinerMake):
    raw_model = "M30S VG10"
    expected_chips = 66
    expected_fans = 2


class M30SVG20(WhatsMinerMake):
    raw_model = "M30S VG20"
    expected_chips = 70
    expected_fans = 2


class M30SVG30(WhatsMinerMake):
    raw_model = "M30S VG30"
    expected_chips = 74
    expected_fans = 2


class M30SVG40(WhatsMinerMake):
    raw_model = "M30S VG40"
    expected_chips = 78
    expected_fans = 2


class M30SVH10(WhatsMinerMake):
    raw_model = "M30S VH10"
    expected_chips = 64
    expected_fans = 2


class M30SVH20(WhatsMinerMake):
    raw_model = "M30S VH20"
    expected_chips = 66
    expected_fans = 2


class M30SVH30(WhatsMinerMake):
    raw_model = "M30S VH30"
    expected_fans = 2


class M30SVH40(WhatsMinerMake):
    raw_model = "M30S VH40"
    expected_chips = 64
    expected_fans = 2


class M30SVH50(WhatsMinerMake):
    raw_model = "M30S VH50"
    expected_chips = 66
    expected_fans = 2


class M30SVH60(WhatsMinerMake):
    raw_model = "M30S VH60"
    expected_fans = 2


class M30SVI20(WhatsMinerMake):
    raw_model = "M30S VI20"
    expected_chips = 70
    expected_fans = 2
