# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------

from pyasic.miners.backends import M3X
from pyasic.miners.models import (
    M30SV10,
    M30SV20,
    M30SV30,
    M30SV40,
    M30SV50,
    M30SV60,
    M30SV70,
    M30SV80,
    M30SVE10,
    M30SVE20,
    M30SVE30,
    M30SVE40,
    M30SVE50,
    M30SVE60,
    M30SVE70,
    M30SVF10,
    M30SVF20,
    M30SVF30,
    M30SVG10,
    M30SVG20,
    M30SVG30,
    M30SVG40,
    M30SVH10,
    M30SVH20,
    M30SVH30,
    M30SVH40,
    M30SVH50,
    M30SVH60,
    M30SVI20,
)


class BTMinerM30SV10(M3X, M30SV10):
    pass


class BTMinerM30SV20(M3X, M30SV20):
    pass


class BTMinerM30SV30(M3X, M30SV30):
    pass


class BTMinerM30SV40(M3X, M30SV40):
    pass


class BTMinerM30SV50(M3X, M30SV50):
    pass


class BTMinerM30SV60(M3X, M30SV60):
    pass


class BTMinerM30SV70(M3X, M30SV70):
    pass


class BTMinerM30SV80(M3X, M30SV80):
    pass


class BTMinerM30SVE10(M3X, M30SVE10):
    pass


class BTMinerM30SVE20(M3X, M30SVE20):
    pass


class BTMinerM30SVE30(M3X, M30SVE30):
    pass


class BTMinerM30SVE40(M3X, M30SVE40):
    pass


class BTMinerM30SVE50(M3X, M30SVE50):
    pass


class BTMinerM30SVE60(M3X, M30SVE60):
    pass


class BTMinerM30SVE70(M3X, M30SVE70):
    pass


class BTMinerM30SVF10(M3X, M30SVF10):
    pass


class BTMinerM30SVF20(M3X, M30SVF20):
    pass


class BTMinerM30SVF30(M3X, M30SVF30):
    pass


class BTMinerM30SVG10(M3X, M30SVG10):
    pass


class BTMinerM30SVG20(M3X, M30SVG20):
    pass


class BTMinerM30SVG30(M3X, M30SVG30):
    pass


class BTMinerM30SVG40(M3X, M30SVG40):
    pass


class BTMinerM30SVH10(M3X, M30SVH10):
    pass


class BTMinerM30SVH20(M3X, M30SVH20):
    pass


class BTMinerM30SVH30(M3X, M30SVH30):
    pass


class BTMinerM30SVH40(M3X, M30SVH40):
    pass


class BTMinerM30SVH50(M3X, M30SVH50):
    pass


class BTMinerM30SVH60(M3X, M30SVH60):
    pass


class BTMinerM30SVI20(M3X, M30SVI20):
    pass
