# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------

from .M50 import (
    BTMinerM50VE30,
    BTMinerM50VG30,
    BTMinerM50VH10,
    BTMinerM50VH20,
    BTMinerM50VH30,
    BTMinerM50VH40,
    BTMinerM50VH50,
    BTMinerM50VH60,
    BTMinerM50VH70,
    BTMinerM50VH80,
    BTMinerM50VJ10,
    BTMinerM50VJ20,
    BTMinerM50VJ30,
)
from .M50S import (
    BTMinerM50SVH10,
    BTMinerM50SVH20,
    BTMinerM50SVH30,
    BTMinerM50SVH40,
    BTMinerM50SVH50,
    BTMinerM50SVJ10,
    BTMinerM50SVJ20,
    BTMinerM50SVJ30,
)
from .M50S_Plus import (
    BTMinerM50SPlusVH30,
    BTMinerM50SPlusVH40,
    BTMinerM50SPlusVJ30,
    BTMinerM50SPlusVK20,
)
from .M50S_Plus_Plus import (
    BTMinerM50SPlusPlusVK10,
    BTMinerM50SPlusPlusVK20,
    BTMinerM50SPlusPlusVK30,
)
from .M53 import BTMinerM53VH30
from .M53S import BTMinerM53SVH30, BTMinerM53SVJ40
from .M53S_Plus import BTMinerM53SPlusVJ30
from .M53S_Plus_Plus import BTMinerM53SPlusPlusVK10
from .M56 import BTMinerM56VH30
from .M56S import BTMinerM56SVH30
from .M56S_Plus import BTMinerM56SPlusVJ30
from .M59 import BTMinerM59VH30
