from pyasic.miners.backends import Auradine
from pyasic.miners.models import AuradineAT2860, AuradineAT2880


class AuradineFluxAT2860(AuradineAT2860, Auradine):
    pass


class AuradineFluxAT2880(AuradineAT2880, Auradine):
    pass
