# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------

from pyasic.miners.backends import M3X
from pyasic.miners.models import (
    M31SPlusV10,
    M31SPlusV20,
    M31SPlusV30,
    M31SPlusV40,
    M31SPlusV50,
    M31SPlusV60,
    M31SPlusV80,
    M31SPlusV90,
    M31SPlusV100,
    M31SPlusVE10,
    M31SPlusVE20,
    M31SPlusVE30,
    M31SPlusVE40,
    M31SPlusVE50,
    M31SPlusVE60,
    M31SPlusVE80,
    M31SPlusVF20,
    M31SPlusVF30,
    M31SPlusVG20,
    M31SPlusVG30,
)


class BTMinerM31SPlusV10(M3X, M31SPlusV10):
    pass


class BTMinerM31SPlusV20(M3X, M31SPlusV20):
    pass


class BTMinerM31SPlusV30(M3X, M31SPlusV30):
    pass


class BTMinerM31SPlusV40(M3X, M31SPlusV40):
    pass


class BTMinerM31SPlusV50(M3X, M31SPlusV50):
    pass


class BTMinerM31SPlusV60(M3X, M31SPlusV60):
    pass


class BTMinerM31SPlusV80(M3X, M31SPlusV80):
    pass


class BTMinerM31SPlusV90(M3X, M31SPlusV90):
    pass


class BTMinerM31SPlusV100(M3X, M31SPlusV100):
    pass


class BTMinerM31SPlusVE10(M3X, M31SPlusVE10):
    pass


class BTMinerM31SPlusVE20(M3X, M31SPlusVE20):
    pass


class BTMinerM31SPlusVE30(M3X, M31SPlusVE30):
    pass


class BTMinerM31SPlusVE40(M3X, M31SPlusVE40):
    pass


class BTMinerM31SPlusVE50(M3X, M31SPlusVE50):
    pass


class BTMinerM31SPlusVE60(M3X, M31SPlusVE60):
    pass


class BTMinerM31SPlusVE80(M3X, M31SPlusVE80):
    pass


class BTMinerM31SPlusVF20(M3X, M31SPlusVF20):
    pass


class BTMinerM31SPlusVF30(M3X, M31SPlusVF30):
    pass


class BTMinerM31SPlusVG20(M3X, M31SPlusVG20):
    pass


class BTMinerM31SPlusVG30(M3X, M31SPlusVG30):
    pass
